//  ver 2.0

function fpx= piir(t0x,forder,freq_file,perror,qx,nfx)
//  t0x:        sample time
//  forder:     IIR filter order
//  freq_file:  spec file name
//  perror:     phase error ( degree )
//  qx:         relative order
//  nfx:        predetermined factor
//  fpx:        IIR filter

  global t0
  t0 = t0x

  arg_num = argn(2)
  qx0 =0
  if arg_num>4 then
    qx0 = qx
  end

  nfn = 1
  nfd = 1
  if arg_num > 5 then
    nfn = numer(nfx)
    nfd = denom(nfx)
  end

  [fp,fmx,fmxp,nl1,xpz,olst] = piir0(forder,freq_file,perror,nfn,nfd,qx0)
  fpx = fmx
endfunction

if exists('def_fderive') == 0 then
 function dgf = fderiv(fz1)
  dgf=derivat(numer(fz1))./denom(fz1)-numer(fz1).*(derivat(denom(fz1)))./denom(fz1).^2
 endfunction
 def_fderive = 1
end

function [LME,LMI,OBJ] = piir2(A0)
 global piir_1
 numer1 = piir_1(1)
 denom1n = piir_1(2)
 forder = piir_1(3)
 fzz    = piir_1(4)

 qq     = piir_1(5)
 w0     = piir_1(6)
 ferr   = piir_1(7)
 farg   = piir_1(8)
 fcomp  = piir_1(9)
 denom1 = piir_1(10)
 SMF    = piir_1(11)
 smooth_f = piir_1(12)

 
 C0 = A0(1)
 lamda = A0(2)

 //
 //  to extract coeff
 //
 fp= 1/z^(forder+1)+full(C0)*fzz*numer1*denom1n
 xx1 = coeff(numer(fp)+z^(2*forder+2-qq))  

 xx = xx1(1,2:(2*forder+2-qq))
 
 [n1,n2]=size(w0)
 
 l_lmi =list() 
 m_lmi =list()
 q1 = zeros(2*forder+1-qq,1)
 q2 = zeros(2*forder+1-qq,1)
 
 for i=1: n1

   for k= -forder : (forder-qq)
     q1(k+forder+1,1) = sin(-k*w0(i,1)+perr(i,1)+farg(i,1))
     q2(k+forder+1,1) = sin(k*w0(i,1)+perr(i,1)-farg(i,1))
   end
   l_lmi(i)=  xx*q1
   m_lmi(i)=  xx*q2
 end

 pe0 =xx(1,1)+1
 pe1 = 1- xx(1,1)



  //   fir case 
 s1= list()


 if forder == degree(denom1) then   // gain is used
   p2 = list()
   s0=full(C0)*fzz*numer1
   for i=1: n1
     s00 = horner(s0/denom1,exp(%i*w0(i,1)))-fcomp(i,1)
     s00 = s00/abs(fcomp(i,1))       //  relative error
     p2(i) = [lamda,real(s00),imag(s00);real(s00),1,0;imag(s00),0,1]
   end
   OBJ=[lamda]
   LMI = list(l_lmi,m_lmi,pe0,pe1,p2,lamda)

 else

//  smooth function
   if size(SMF,'*') <> 0 then
    //  smooth function
     st1 = full(C0)*SMF
   
     [n1,n2] = size(SMF) 
     if smooth_f > 0 then
       lm1 = lamda
       OBJ=[lamda]
     else
       lm1 = abs(smooth_f)
       OBJ=[]
     end

     for i = 1:n2
       S0 = eye(3,3)
       S0(1,1) = lm1
       S0(1,2) = real(st1(1,i))
       S0(2,1) = real(st1(1,i))
       S0(1,3) = imag(st1(1,i))
       S0(3,1) = imag(st1(1,i))
       s1(i) = S0
   end
   
     pe2 =xx(1,1)-1
 
     LMI = list(l_lmi,m_lmi,pe2,s1,lamda)
   else
     OBJ=[]
     LMI = list(l_lmi,m_lmi,pe0,pe1,lamda)
   end

 end

 LME=[]
 
endfunction


function [fpx,fmx,fmxp,nl1,xpz,olst] = piir0(forder,freq_file,perror,numer1x,denom1x,qqx,smooth_fx,lmt_polx)
//  main prg
// 
//  get IIR from phase data
//  Gain is used if the order of denominator is equal to denom1.
//  After getting good phase, gain adjust routine is called. ( see forder )
//
//  input
//      forder:    order of iir  if minus, the gain routine starts
//      freq_file: file in which
//                    freq  gain  phase
//      perror:  phase error    [f1,p1;f2,p2..] or p0
//      numer1:  default polynomial of numerator
//      denom1:  default polynomial of numerator
//      qqx:   if >= 0  degree(denominator)-degree(numerator),  gain adjust is not applied if mini phase
//      smooth_f: =1 smooth function active,  <0  |smooth_f| > dw,  condition x0 > 1 is added
//      lmt_pol: poles > 1-lmt_pol are considered unstable 
// 
//   output
//      fpx:   forder answer
//      fmx:   answer for gain adjust
//      fmxp:  function before gain adjust  = z^nl1*fmxp
//      nl1:   function before gain adjust  = z^nl1*fmxp
//      xpz:   xpz(1) -> zeros of fmx,  xpz(2) -> poles of fmx  
//             check adjacent zeros/poles
//

 arg_num = argn(2)

 global t0

 last_arg = 3

 numer1 = 1
 if arg_num > last_arg then
   numer1 = numer1x
 end

 denom1 = 1
 if arg_num > last_arg+1 then
   denom1 = denom1x
 end

 qq = 0
 k0_flag=1
 if arg_num > last_arg+2 then
   if qqx  >=0 then
     qq = qqx
   else
     qq = 0
     k0_flag = 0
   end
 end

 smooth_f = 0
 if arg_num > last_arg+3 then
   smooth_f = smooth_fx
 end

 lmt_pol = 0
 if arg_num > last_arg+4 then
   lmt_pol = lmt_polx
 end


 run_flag = 0
 if forder < 0 then 
   run_flag = 1
   forder = abs(forder)
 end

 fd=mopen(freq_file,'r')
 [freqnum,fw,fabs,farg]=mfscanf(-1,fd,"%e %e %e")
 mclose(fd)

  
 fcomp = fabs.*exp(%i*farg/180*%pi)
 farg = imag(log(fcomp))
 //farg = farg/180*%pi
 [n1,m1] = size(perror)

 //  phase error

 perr=[]
 if m1 == 1 then 
   perr = perror(1,1)*ones(fw)
 else
   fl = perror(n1,1)    // last freq

   if fl < 1/t0/2 then  // add data till the freq. higher than Nyquist
      perror= [perror;1/t0,perror(n1,2)]
      n1 = n1+1
   end

   perr = zeros(fw)
   for i=1:n1
      fx = perror(i,1)*ones(fw)   // get freq
      if i <> 1 then
        fx1 = perror(i-1,1)*ones(fw)
      else
        fx1 = -1*ones(fw)
      end
      fn = (fw <= fx) & ( fw > fx1)  // set T or F
      perr = perr + perror(i,2)*fn
   end
 end

 perr = perr/180*%pi

 A0 = ones(1:(2*forder+1-qq-(degree(numer1))-(degree(denom1))))
 // if forder == degree(denom1) then  // fir case
 popt = 1 
 A0l = list(A0,popt)
 
 // end



 coef_den = coeff(denom1)
 
 fzz= z^(0:-1:-degree(denom1))
 denom1n = coef_den*fzz'

 fzz = z^(-(forder-degree(denom1)):(forder-qq-degree(numer1)))
 fzz =fzz'

 w0 = fw*t0*2*%pi



//  smooth function 
//  F = fa1 + fp1*(c0*z^(-starti) + c1*z^(-starti-1) + ..
//  E = log ( log_fun ) + F

 SMF = []

 if smooth_f <> 0 then
   ns = 100
   dw = 0.02

   ws0 = 0.001*1/t0/2
   wsn = 0.999*1/t0/2/(1+dw)

   wn = (wsn-ws0)*(0: ns )/ns + ws0
   wn0= 2*%pi*t0*wn

   z0 = exp(%i*wn0)
   z1 = exp(%i*wn0*(1+dw))
 
   tf1 = fderiv(fzz)*%i*z
 
//  F1 - F0   
   iw = inv(diag(wn0*dw))
 
   dwx = (horner(fzz,z1)-horner(fzz,z0))*iw - horner(tf1,z0)
// take real only

   SMF = dwx  //  E(i,1) = SMF(i,:)*C0
 end


 global piir_1
 piir_1 = list( numer1, denom1n, forder, fzz, qq, w0, perr, farg ,fcomp, denom1,SMF,smooth_f)

 Mbound = 1e3
 abstol = 1e-13
 nu = 1
 maxiters = 1000
 reltol = 1e-10

 options = [Mbound,abstol,nu,maxiters,reltol]
 fans = lmisolver(A0l,piir2,options)
 
 if smooth_f == 1 then
   disp('lamda=')
   disp(fans(2))
 end

 [n1,n2]=size(fzz)
 fz0= z^((n1-1):-1:0)
 fp= fans(1)*fz0'

////    select poles which are less than 1-lmt_pol
//      sort by abs of poles and take the poles 
//             from the first one to the forder'th one.
//      if number of complex poles is odd, relace the forder'th one
//             to a real pole.

 fpx = 1                 // prepare output
 fmx = 1
 fmxp = 1
 
 num_pol = forder-degree(denom1)
 
 if num_pol <> 0 then    // select poles
   af = roots(fp)

   av = abs(af)
   [qabs1,k1]=gsort(av,'g','i')    // decreasing order 

   afv = af(k1)                    // sorted poles

   ak =afv(1:num_pol,1)            //  get small pole

   l1 = 0                          // flag for failure
   if abs(ak(num_pol,1)) > (1-lmt_pol) then
     l1 = 1
   else
     aimg = (imag(ak) == 0)        // check real number
     one1 = 1
     sumone = num_pol-sum(one1*aimg)  // complex number
     l1 = modulo(sumone,2)
 
     if l1 <> 0 then               // number of complex should be even
       for i= 1:size(afv,'*')
    
        if imag(afv(i,1)) == 0 then    // found real number
          if abs(afv(i,1)) < (1-lmt_pol) then 
            if i<  num_pol then
              pafv = afv(i,1)        // replace last one
              xafv = afv(num_pol+1,1)
              afv(i,1)=xafv
              afv(num_pol+1,1)=pafv
             
            else
              pafv = afv(i,1)        // replace last one
              xafv = afv(num_pol,1)
              afv(i,1)=xafv
              afv(num_pol,1)=pafv
            end
            ak = afv(1:num_pol,1)   // replace last one
            l1 = 0 
            break
          else
            break
          end
        end
       end
     end
   end


   fdenom = 1
   fnumer = 1
   fden0 = 1
   fnum0 = 1

// get forder answer 
   if l1 <> 0 then
     disp ( ' no n order answer, type abort' )
     disp (afv)
     pause
   else
     fden0 = poly(ak,'z','roots')
     fdenom = fden0*denom1
     fnumer = numer1
   
     if (num_pol+1) <= size(afv,'*') then
       am = afv((num_pol+1):size(afv,'*'),1)
       amv = ones(am)./am
       fnum0 = poly(amv,'z','roots')
       fnumer = fnum0*numer1
     end
   end

//  get z^m*func answer. Take all poles less than 1

   la1 = qabs1 < (1-lmt_pol)   // number of poles less then 1
   nla1 = sum(1*la1)

   fdenom1 = fdenom
   fnumer1 = fnumer

   fden00 = fden0
   fnum00 = fnum0

   if nla1 > num_pol then   // if number of zero is larger than required pole
     fden00 = poly(afv(1:nla1,1),'z','roots')
     fdenom1 = fden00*denom1
     as1 = afv(num_pol+1:nla1,1)
     qx = imag(as1)==0
     fg1 =1
     if size(as1(qx),'*') <> 0 then
       fg1 = prod(sign(as1(qx)))
     end
     ax1 =(-1)^sum(nla1-num_pol)*fg1           //  sign(1/(1/1-a))=sign(1/(1-1/a)(a-1)/a/(1-a))
     fnum00 = ax1
     fnumer1 = numer1*fnum00
     if (nla1+1) <= size(afv,'*') then
       am = afv((nla1+1):size(afv,'*'),1)
       amv = ones(am)./am
       fnumer1 = poly(amv,'z','roots')*fnumer1
       fnum00 = poly(amv,'z','roots')*fnum00
     end
   end

   nl1 = nla1-num_pol
 
   fpx = syslin(t0,real(fnumer)/real(fdenom))
   fmx = syslin(t0,real(fnumer1*z^nl1)/real(fdenom1))
   fmxp =syslin(t0,real(fnumer1)/real(fdenom1))

   fpx00 = syslin(t0,real(fnum0)/real(fden0))
   fmx00 = syslin(t0,real(fnum00*z^nl1)/real(fden00))
   fmxp00 =syslin(t0,real(fnum00)/real(fden00))

 else
   fpx = syslin(t0,fans(1)*fzz*numer1/denom1)
   fmx = fpx
   fmxp = fpx

   fpx00 = fans(1)*fzz
   fmx00 = fpx00
   fmxp00 =fpx00

end

 // gain adjust for forder answer
 sflg = 1
 if forder <> degree(denom1) then
   [f1,v1]=repfreq(fpx,fw)
   if k0_flag == 1 then
     g1 =abs(v1)
     g2 =abs(fcomp.')
     k0 = sum(g2.*g1)/sum(g1.*g1)
   else
     k0 = 1
   end
   fpx = k0*fpx
   fpx00 = k0*fpx00

   fs0 = repfreq(fpx,fw)
   adfs0 = sum(abs(fs0-fcomp.'))
   sbfs0 = sum(abs(fs0+fcomp.'))
   if sbfs0 < adfs0 then            //  check whether negative or not
     fpx = -fpx
     sflg = -1
     disp('warning the target is negative!!!!')
   end

 end
 
 [f1,v2]=repfreq(fmx,fw)
 
 if k0_flag == 1 then
   g1 =abs(v2)
   g2 =abs(fcomp.')
   k00 = sum(g2.*g1)/sum(g1.*g1)
 else
   k00 = 1
 end  
 k00 = k00*sflg
 fmx = k00*fmx
 fmxp = k00*fmxp

 fmx00 = k00*fmx00
 fmxp00 = k00*fmxp00
 ppx = 1
 //
 xpz = []

 //  gain adjust routine
 // 
 s1 ='z^m*IIR'
 if (nl1 <> 0) & (run_flag == 1) then
   disp ( 'starting gain adjustment')
   n_skp = floor(size(fw,'*')/50)
   if n_skp == 0 then
     n_skp = 1
 end
   [pxx,xpz,ppx] = get_gain ( fmxp, nl1,freq_file, 0,n_skp )
   fmx = pxx
   s1 = 'gain adjusted 2nd output'
 end
//

 olst = list()

 olst(1)  = fpx00
 olst(2)  = fmxp00*ppx  
 // draw graph
 
 scf(0)
 clf(0)
 
 if forder == degree(denom1) then
   [f1,v1]=repfreq(fpx,fw)
   bode(fw',[fcomp.';v1],['gain adjusted';'target'])
 else
   [f1,v1]=repfreq(fpx,fw)
   [f1,v2]=repfreq(fmx,fw)
 
//   bode(fw',[fcomp.';v1;v2],['target';'IIR  1st output';s1])
   bode(fw',[fcomp.';v2],['IIR';'target'])  // for release
 end  
endfunction


function [LME,LMI,OBJ] = pgain2(A0)
 global pgain_1
 g_flag = pgain_1(1)
 fws    = pgain_1(2)
 tabs   = pgain_1(3)
 pole1  = pgain_1(4)
 aerr   = pgain_1(5)
 gwt    = pgain_1(6)

 C0 = A0(1)
 lamda = A0(2)
 
 norder = size(full(C0),'*')-1
 fz = z^(0:norder)
 fzi = z^(-(0:norder))

 p1 = list()

 if g_flag == 0 | g_flag == 2 then 

//  relative error
   n1 = size(fws,'*')
   for i=1: n1

     w0 = fws(i,1)*2*%pi*t0
     c1 = real(horner(fz,exp(%i*w0)))
     sc0 = (sum(c1.*C0)-tabs(1,i))/tabs(1,i)
 
     p1(i) = [lamda,sc0;sc0,1/gwt(i,1)]
   end

 elseif g_flag == 1 then
//  sum of absolute error

   n1 = size(fws,'*')
   Cm = eye(n1+1,n1+1)
   Cm(1,1) = lamda
   for i=1: n1
     w0 = fws(i,1)*2*%pi*t0
     c1 = real(horner(fz,exp(%i*w0)))
     sc0 = (sum(c1.*C0)-tabs(1,i))
     Cm(1,i+1) = sc0
     Cm(i+1,1) = sc0
     Cm(i+1,i+1) = 1/gwt(i,1)
   end
   p1(1)= Cm
   
 end
 
 lme_i =list()
 p2 = list()
 if g_flag == 2 then
   fzp = horner(z^norder*fz',pole1)
   fzip = horner(z^norder*fzi',pole1)
   c1 = full(C0)*(fzp+fzip)*0.5
   n1 = size(c1,'*')
   i_prt=1
   for i = 1:n1   
    lme_i(i_prt) = real(c1(1,i))     
    i_prt = i_prt+1
    if imag(pole1(1,i)) <> 0 then
      lme_i(i_prt) = imag(c1(1,i))     
      i_prt = i_prt+1
    end
  //   p2(i) = [aerr,real(c1(1,i)),imag(c1(1,i));real(c1(1,i)),1,0;imag(c1(1,i)),0,1]
   end
 end

 LMI = list(p1,p2)
 LME=lme_i

 OBJ=[lamda]
endfunction


//
//     gain adjust routine 
//
//   set d_flagx = 1 if run from console.
//

function [px,xzp,pol0] = get_gain ( fmxp, nlx,freq_file, g_flagx,n_skipx, gweightx, d_flagx, pole1x, aerrx )
//
//  input
//     fmxp:      fmx = z^nlx*fmxp
//     nlx:       z^nlx
//     freq_file: file in which
//                    freq  gain  phase
//     g_flagx:    0 relative error, 1 sum of abs error, 2 pole delete
//     n_skipx:   data is taken every n_skipx
//     gweightx:  weight    [f1,w1;f2,w2..] or w0
//     d_flagx:    0 no disp, 1 display bode
//     pole1:     poles to be deleted => [ n1, n2 ]  p(n1),p(n2) are deleted
//                 poles from xpz(2)
//     aerr:      allowable error for cancelling poles
//
//  poles of fmxp can be deleted by zero if g_flag == 2
//

 arg_num = argn(2)

 last_arg = 3

 g_flag = 0
 if arg_num > last_arg then
   g_flag = g_flagx
 end

 n_skip = 1
 if arg_num > last_arg+1 then
   n_skip = n_skipx
 end

 gweight=1
 if arg_num > last_arg+2 then
   gweight = gweightx
 end

 d_flag = 0
 if arg_num > last_arg+3 then
   d_flag= d_flagx
 end


 npole1 = []
 if arg_num > last_arg+4 then
   npole1 = pole1x
   if g_flag <> 2 then
     disp('g_flag should be 2, type abort')
     pause
   end
 end
 rr1 = roots(denom(fmxp))
 rr1 = rr1.'
 pole1 = rr1(1,npole1)
 
 aerr = 1e-9
 if arg_num > last_arg+5 then
   aerr = aerrx
 end

 fd=mopen(freq_file,'r')
 [freqnum,fw,fabs,farg]=mfscanf(-1,fd,"%e %e %e")
 mclose(fd)

 //  weight

 gwt=[]
 [n1,m1] = size(gweight)
 if m1 == 1 then 
   gwt = gweight(1,1)*ones(fw)
 else
   fl = gweight(n1,1)    // last freq

   if fl < 1/t0/2 then  // add data till the freq. higher than Nyquist
      gweight= [gweight;1/t0,gweight(n1,2)]
      n1 = n1+1
   end

   gwt = zeros(fw)
   for i=1:n1
      fx = gweight(i,1)*ones(fw)   // get freq
      if i <> 1 then
        fx1 = gweight(i-1,1)*ones(fw)
      else
        fx1 = -1*ones(fw)
      end
      fn = (fw <= fx) & ( fw > fx1)  // set T or F
      gwt = gwt + gweight(i,2)*fn
   end
 end


 fws = fw(1:n_skip:size(fw,'*'),1)
 fabss = fabs(1:n_skip:size(fabs,'*'),1)
 gwt = gwt(1:n_skip:size(gwt,'*'),1)
 
 zmf = repfreq(fmxp*z^nlx,fws')

 tabs = fabss'./abs(zmf)

 global pgain_1

 pgain_1= list(g_flag, fws, tabs, pole1, aerr, gwt)

 A0 = ones(1,nlx+1)
 lamda = 1
  
 A0l = list(A0,lamda)
 
 Mbound = 1e3
 abstol = 1e-13
 nu = 1
 maxiters = 1000
 reltol = 1e-10

 options = [Mbound,abstol,nu,maxiters,reltol]
 fans = lmisolver(A0l,pgain2,options)


 fz = (z^(0:nlx)+z^(-(0:nlx)))/2

 fsum = fans(1)*fz'

 if sum(fans(1)) ==  0 then
  disp('no answer with this condition')

  pause
 end

 px0 = z^nlx*fsum

 pol0 = px0       //   return 

 px = syslin(t0,px0*fmxp)
 
 
 if g_flag == 2 then
   xzero = roots(numer(px))
   xpole = roots(denom(fmxp))
   fzd = 1
   n1 = degree(denom(fmxp))
   for i = 1:n1
     q1 = (npole1 == ones(npole1)*i)   // check specified poles
     q2 = sum(1*q1)
     if q2 == 0 then
       fzd = fzd*(z-xpole(i,1))        // pick up not specified poles
     end
   end

   n1 = degree(numer(px))
   fzn = 1
   for i = 1:n1
     q1 = (abs(xzero(i,1)*ones(pole1) - pole1) < 1e-3 ) 
     q2 = sum(1*q1)
     if q2 == 0 then
       fzn = fzn*(z-xzero(i,1))
     end
   end
   c2 = coeff(numer(px))
   fzn = fzn*c2(1,n1+1)
   
   if (degree(numer(px))-degree(fzn)) == (degree(denom(fmxp))-degree(fzd)) then
     px = syslin(t0,real(fzn)/real(fzd))
     pol0 = fzn
     disp ( 'the order is reduced')
   end
  
 end
 
 if d_flag <> 0 then
   [f1,v1]=repfreq(px,fw)
   fcomp = fabs.*exp(%i*farg/180*%pi)
   bode(fw',[fcomp.';v1],['target';'gain adjusted'])
 end


 xzp = list(roots(numer(px)),roots(denom(px)))
endfunction


function read_tgt(file1)
//
// draw target
//
 aaq = read(file1,-1,3)

 ya = aaq(:,2).*exp(%i*aaq(:,3)/180*%pi)
 clf()
 bode(aaq(:,1),ya)
endfunction

